@@ -305,7 +305,7 @@ define_destination_uri (EphyDownload *download, const char *suggested_filename)
   }
 
   if (suggested_filename != NULL) {
-    dest_name = g_strdup (suggested_filename);
+    dest_name = ephy_sanitize_filename (g_strdup (suggested_filename));
   } else {
     dest_name = ephy_file_tmp_filename ("ephy-download-XXXXXX", NULL);
   }
@@ -670,7 +670,8 @@ ephy_download_do_download_action (EphyDownload *download,
         break;
       case EPHY_DOWNLOAD_ACTION_OPEN:
         LOG ("ephy_download_do_download_action: open");
-        ret = ephy_file_launch_handler (NULL, destination, priv->start_time);
+        ret = ephy_embed_shell_launch_handler (ephy_embed_shell_get_default (), 
+                                               destination, NULL, priv->start_time);
         break;
       case EPHY_DOWNLOAD_ACTION_NONE:
         LOG ("ephy_download_do_download_action: none");
@@ -715,7 +716,7 @@ ephy_download_dispose (GObject *object)
     priv->widget = NULL;
   }
 
-  ephy_embed_shell_remove_download (embed_shell, download);
+  ephy_embed_shell_remove_download (ephy_embed_shell_get_default (), download);
 
   G_OBJECT_CLASS (ephy_download_parent_class)->dispose (object);
 }
@@ -955,7 +956,7 @@ download_finished_cb (WebKitDownload *wk_download,
   else
     ephy_download_do_download_action (download, priv->action);
 
-  ephy_embed_shell_remove_download (embed_shell, download);
+  ephy_embed_shell_remove_download (ephy_embed_shell_get_default (), download);
 }
 #else
 static void
@@ -980,11 +981,11 @@ download_status_changed_cb (GObject *object,
       ephy_download_do_download_action (download, priv->action);
     }
 
-    ephy_embed_shell_remove_download (embed_shell, download);
+    ephy_embed_shell_remove_download (ephy_embed_shell_get_default (), download);
   } else if (status == WEBKIT_DOWNLOAD_STATUS_CANCELLED ||
              status == WEBKIT_DOWNLOAD_STATUS_ERROR) {
   } else if (status == WEBKIT_DOWNLOAD_STATUS_STARTED) {
-    ephy_embed_shell_add_download (embed_shell, download);
+    ephy_embed_shell_add_download (ephy_embed_shell_get_default (), download);
   }
 }
 #endif
@@ -1090,7 +1091,7 @@ ephy_download_new_for_download (WebKitDownload *download)
 
 #ifdef HAVE_WEBKIT2
   /* In WebKit2 the download has already started */
-  ephy_embed_shell_add_download (embed_shell, ephy_download);
+  ephy_embed_shell_add_download (ephy_embed_shell_get_default (), ephy_download);
 #endif
 
   return ephy_download;
