@@ -31,6 +31,7 @@
 #include "ephy-encodings.h"
 #include "ephy-file-helpers.h"
 #include "ephy-history-service.h"
+#include "ephy-profile-utils.h"
 #include "ephy-snapshot-service.h"
 
 #include <glib/gi18n.h>
@@ -124,7 +125,7 @@ ephy_embed_shell_get_global_history_service (EphyEmbedShell *shell)
   if (shell->priv->global_history_service == NULL) {
     char *filename;
 
-    filename = g_build_filename (ephy_dot_dir (), "ephy-history.db", NULL);
+    filename = g_build_filename (ephy_dot_dir (), EPHY_HISTORY_FILE, NULL);
     shell->priv->global_history_service = ephy_history_service_new (filename);
     g_free (filename);
     g_return_val_if_fail (shell->priv->global_history_service, NULL);
@@ -323,7 +324,7 @@ ephy_embed_shell_class_init (EphyEmbedShellClass *klass)
   object_properties[PROP_MODE] =
     g_param_spec_enum ("mode",
                        "Mode",
-                       "The  global mode f  or this instance of Epiphany .",
+                       "The  global mode for this instance of Epiphany .",
                        EPHY_TYPE_EMBED_SHELL_MODE,
                        EPHY_EMBED_SHELL_MODE_BROWSER,
                        G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK |
@@ -608,3 +609,47 @@ ephy_embed_shell_get_mode (EphyEmbedShell *shell)
   
   return shell->priv->mode;
 }
+
+/**
+ * ephy_embed_shell_launch_application:
+ * @shell: an #EphyEmbedShell
+ * @file: a #GFile to open
+ * @mime_type: the mime type of @file or %NULL
+ * @user_time: user time to prevent focus stealing
+ * 
+ * Tries to open @file with the right application, making sure we will
+ * not call ourselves in the process. This is needed to avoid
+ * potential infinite loops when opening unknown file types.
+ * 
+ * Returns: %TRUE on success
+ **/
+gboolean
+ephy_embed_shell_launch_handler (EphyEmbedShell *shell,
+                                 GFile *file,
+                                 const char *mime_type,
+                                 guint32 user_time)
+{
+  GAppInfo *app;
+  GList *list = NULL;
+  gboolean ret = FALSE;
+
+  g_return_val_if_fail (EPHY_IS_EMBED_SHELL (shell), FALSE);
+  g_return_val_if_fail (file || mime_type, FALSE);
+
+  app = ephy_file_launcher_get_app_info_for_file (file, mime_type);
+
+  /* Do not allow recursive calls into the browser, they can lead to
+   * infinite loops and they should never happen anyway. */
+
+  /* FIXME: eventually there should be a nice and safe way of getting
+   * the app ID from the GApplication itself, but for now let's
+   * hardcode the .desktop file name and use it here. */
+  if (!app || g_strcmp0 (g_app_info_get_id (app), "epiphany.desktop") == 0)
+    return ret;
+
+  list = g_list_append (list, file);
+  ret = ephy_file_launch_application (app, list, user_time, NULL);
+  g_list_free (list);
+
+  return ret;
+}
