@@ -249,7 +249,7 @@ resource_request_starting_cb (WebKitWebView *web_view,
                               WebKitNetworkResponse *response,
                               EphyEmbed *embed)
 {
-  EphyAdBlockManager *adblock_manager = EPHY_ADBLOCK_MANAGER (ephy_embed_shell_get_adblock_manager (embed_shell));
+  EphyAdBlockManager *adblock_manager = EPHY_ADBLOCK_MANAGER (ephy_embed_shell_get_adblock_manager (ephy_embed_shell_get_default ()));
   const char *uri = webkit_network_request_get_uri (request);
 
   /* FIXME: How do we implement the other CHECK_TYPEs?  Perhaps we
@@ -505,6 +505,11 @@ ephy_embed_class_init (EphyEmbedClass *klass)
   object_class->get_property = ephy_embed_get_property;
   widget_class->grab_focus = ephy_embed_grab_focus;
 
+/**
+ * EphyEmbed:overview-mode:
+ *
+ * If %TRUE activates the overview mode in this #EphyEmbed.
+ **/
   g_object_class_install_property (object_class,
                                    PROP_OVERVIEW_MODE,
                                    g_param_spec_boolean ("overview-mode",
@@ -875,8 +880,6 @@ ephy_embed_constructed (GObject *object)
   scrolled_window = GTK_WIDGET (priv->scrolled_window);
 #endif
   overlay = gtk_overlay_new ();
-  gtk_style_context_add_class (gtk_widget_get_style_context (overlay),
-                               GTK_STYLE_CLASS_OSD);
 
   gtk_widget_add_events (overlay, 
                          GDK_ENTER_NOTIFY_MASK |
@@ -905,6 +908,8 @@ ephy_embed_constructed (GObject *object)
   gtk_overlay_add_overlay (GTK_OVERLAY (overlay), priv->floating_bar);
 
   priv->progress = gtk_progress_bar_new ();
+  gtk_style_context_add_class (gtk_widget_get_style_context (priv->progress),
+                               GTK_STYLE_CLASS_OSD);
   gtk_widget_set_halign (priv->progress, GTK_ALIGN_FILL);
   gtk_widget_set_valign (priv->progress, GTK_ALIGN_START);
   gtk_overlay_add_overlay (GTK_OVERLAY (overlay), priv->progress);
