@@ -3,6 +3,7 @@
  *  Copyright © 2002 Jorn Baayen
  *  Copyright © 2003, 2004 Marco Pesenti Gritti
  *  Copyright © 2004, 2005, 2006 Christian Persch
+ *  Copyright © 2012 Igalia S.L.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -49,9 +50,6 @@
  * atomically.
  */
 
-#define EPHY_UUID		"0d82d98f-7079-401c-abff-203fcde1ece3"
-#define EPHY_UUID_ENVSTRING	EPHY_UUID_ENVVAR "=" EPHY_UUID
-
 #define DELAY_MAX_TICKS	64
 #define INITIAL_TICKS	2
 
@@ -306,19 +304,9 @@ ephy_file_helpers_init (const char *profile_dir,
 			EphyFileHelpersFlags flags,
 			GError **error)
 {
-	const char *uuid;
+	gboolean ret = TRUE;
 	gboolean private_profile;
-
-	/* See if we've been calling ourself, and abort if we have */
-	uuid = g_getenv (EPHY_UUID_ENVVAR);
-	if (uuid && strcmp (uuid, EPHY_UUID) == 0)
-	{
-		g_warning ("Self call detected, exiting!\n");
-		exit (1);
-	}
-
-	/* Put marker in env */
-	g_setenv (EPHY_UUID_ENVVAR, EPHY_UUID, TRUE);
+	gboolean steal_data_from_profile;
 
 	ephy_file_helpers_error_quark = g_quark_from_static_string ("ephy-file-helpers-error");
 
@@ -329,8 +317,9 @@ ephy_file_helpers_init (const char *profile_dir,
 
 	keep_directory = flags & EPHY_FILE_HELPERS_KEEP_DIR;
 	private_profile = flags & EPHY_FILE_HELPERS_PRIVATE_PROFILE;
+	steal_data_from_profile = flags & EPHY_FILE_HELPERS_STEAL_DATA;
 
-	if (private_profile && profile_dir != NULL)
+	if (private_profile && profile_dir != NULL && !steal_data_from_profile)
 	{
 		dot_dir = g_strdup (profile_dir);
 	}
@@ -359,9 +348,46 @@ ephy_file_helpers_init (const char *profile_dir,
 	}
 
 	if (flags & EPHY_FILE_HELPERS_ENSURE_EXISTS)
-		return ephy_ensure_dir_exists (ephy_dot_dir (), error);
-	else
-		return TRUE;
+		ret = ephy_ensure_dir_exists (ephy_dot_dir (), error);
+
+	if (steal_data_from_profile && profile_dir)
+	{
+		int i;
+		char *files_to_copy[] = { EPHY_HISTORY_FILE, EPHY_BOOKMARKS_FILE };
+		
+		for (i = 0; i < G_N_ELEMENTS (files_to_copy); i++)
+		{
+			char *filename;
+			GError *error = NULL;
+			GFile *source, *destination;
+
+			filename = g_build_filename (profile_dir,
+						     files_to_copy[i],
+						     NULL);
+			source = g_file_new_for_path (filename);
+			g_free (filename);
+			
+			filename = g_build_filename (dot_dir,
+						     files_to_copy[i],
+						     NULL);
+			destination = g_file_new_for_path (filename);
+			g_free (filename);
+
+			g_file_copy (source, destination,
+				     G_FILE_COPY_OVERWRITE,
+				     NULL, NULL, NULL, &error);
+			if (error)
+			{
+				printf("Error stealing file %s from profile: %s\n", files_to_copy[i], error->message);
+				g_error_free (error);
+			}
+				
+			g_object_unref (source);
+			g_object_unref (destination);
+		}
+	}
+
+	return ret;
 }
 
 static void
@@ -417,8 +443,6 @@ ephy_file_helpers_shutdown (void)
 		g_free (tmp_dir);
 		tmp_dir = NULL;
 	}
-
-	g_unsetenv (EPHY_UUID_ENVVAR);
 }
 
 /**
@@ -825,6 +849,48 @@ ephy_file_launch_desktop_file (const char *filename,
 	return ret;
 }
 
+GAppInfo *
+ephy_file_launcher_get_app_info_for_file (GFile *file,
+					  const char *mime_type)
+{
+	GAppInfo *app = NULL;
+
+	g_return_val_if_fail (file || mime_type, FALSE);
+
+	if (mime_type != NULL)
+	{
+		app = g_app_info_get_default_for_type (mime_type,
+						       FALSE);
+	}
+	else
+	{
+		GFileInfo *file_info;
+		char *type;
+
+		/* Sniff mime type and check if it's safe to open */
+		file_info = g_file_query_info (file,
+					       G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE,
+					       0, NULL, NULL);
+		if (file_info == NULL)
+		{
+			return FALSE;
+		}
+		type = g_strdup (g_file_info_get_content_type (file_info));
+		
+		g_object_unref (file_info);
+
+		if (type != NULL && type[0] != '\0' &&
+		    ephy_file_check_mime (type) == EPHY_MIME_PERMISSION_SAFE)
+		{
+			/* FIXME rename tmp file to right extension ? */
+			app = g_app_info_get_default_for_type (type, FALSE);
+		}
+		g_free (type);
+	}
+
+	return app;
+}
+
 /**
  * ephy_file_launch_handler:
  * @mime_type: the mime type of @file or %NULL
@@ -846,35 +912,7 @@ ephy_file_launch_handler (const char *mime_type,
 
 	g_return_val_if_fail (file != NULL, FALSE);
 
-	if (mime_type != NULL)
-	{
-		app = g_app_info_get_default_for_type (mime_type,
-						       FALSE);
-	}
-	else
-	{
-		GFileInfo *file_info;
-		char *type;
-
-		/* Sniff mime type and check if it's safe to open */
-		file_info = g_file_query_info (file,
-					       G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE,
-					       0, NULL, NULL);
-		if (file_info == NULL) {
-			return FALSE;
-		}
-		type = g_strdup (g_file_info_get_content_type (file_info));
-		
-		g_object_unref (file_info);
-
-		if (type != NULL && type[0] != '\0' &&
-		    ephy_file_check_mime (type) == EPHY_MIME_PERMISSION_SAFE)
-		{
-			/* FIXME rename tmp file to right extension ? */
-			app = g_app_info_get_default_for_type (type, FALSE);
-		}
-		g_free (type);
-	}
+	app = ephy_file_launcher_get_app_info_for_file (file, mime_type);
 
 	if (app != NULL)
 	{
@@ -884,8 +922,6 @@ ephy_file_launch_handler (const char *mime_type,
 		ret = ephy_file_launch_application (app, list, user_time, NULL);
 		g_list_free (list);
 	}
-	else
-		ret = FALSE;
 
 	return ret;
 }
