@@ -108,6 +108,7 @@ typedef enum
 #define EPHY_PREFS_WEB_IMAGE_ANIMATION_MODE  "image-animation-mode"
 #define EPHY_PREFS_WEB_DEFAULT_ENCODING      "default-encoding"
 #define EPHY_PREFS_WEB_DO_NOT_TRACK          "do-not-track"
+#define EPHY_PREFS_WEB_ENABLE_ADBLOCK        "enable-adblock"
 
 #define EPHY_PREFS_SCHEMA                         "org.gnome.Epiphany"
 #define EPHY_PREFS_USER_AGENT                     "user-agent"
@@ -121,7 +122,6 @@ typedef enum
 #define EPHY_PREFS_MANAGED_NETWORK                "managed-network"
 #define EPHY_PREFS_ENABLE_SMOOTH_SCROLLING        "enable-smooth-scrolling"
 #define EPHY_PREFS_ENABLE_CARET_BROWSING          "enable-caret-browsing"
-#define EPHY_PREFS_ENABLED_EXTENSIONS             "enabled-extensions"
 #define EPHY_PREFS_INTERNAL_VIEW_SOURCE           "internal-view-source"
 #define EPHY_PREFS_RESTORE_SESSION_POLICY         "restore-session-policy"
 
