@@ -47,7 +47,6 @@
 #include <glib/gi18n.h>
 #include <glib/gstdio.h>
 #include <gnome-keyring.h>
-#include <libsoup/soup-gnome.h>
 #include <sys/stat.h>
 #include <sys/types.h>
 
@@ -64,13 +63,13 @@ static char *profile_dir = NULL;
 typedef void (*EphyProfileMigrator) (void);
 
 static gboolean
-profile_dir_exists ()
+profile_dir_exists (void)
 {
   return g_file_test (ephy_dot_dir (), G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR);
 }
 
 static void
-migrate_cookies ()
+migrate_cookies (void)
 {
   const char *cookies_file_sqlite = "cookies.sqlite";
   const char *cookies_file_txt = "cookies.txt";
@@ -105,7 +104,7 @@ migrate_cookies ()
     SoupCookieJar *txt, *sqlite;
 
     txt = soup_cookie_jar_text_new (src_txt, TRUE);
-    sqlite = soup_cookie_jar_sqlite_new (dest, FALSE);
+    sqlite = soup_cookie_jar_db_new (dest, FALSE);
     cookies = soup_cookie_jar_all_cookies (txt);
 
     for (p = cookies; p; p = p->next) {
@@ -335,7 +334,7 @@ parse_and_decrypt_signons (const char *signons,
 #endif
 
 static void
-migrate_passwords ()
+migrate_passwords (void)
 {
 #ifdef ENABLE_NSS
   char *dest, *contents, *gecko_passwords_backup;
@@ -383,7 +382,7 @@ migrate_passwords ()
 }
 
 static void
-migrate_passwords2 ()
+migrate_passwords2 (void)
 {
 #ifdef ENABLE_NSS
   char *dest, *contents;
@@ -549,7 +548,7 @@ static GMarkupParser history_parse_funcs =
 };
 
 static void
-migrate_history ()
+migrate_history (void)
 {
   GFileInputStream *input;
   GMarkupParseContext *context;
@@ -559,11 +558,11 @@ migrate_history ()
   char buffer[1024];
   HistoryParseData parse_data;
 
-  gchar *temporary_file = g_build_filename (ephy_dot_dir (), "ephy-history.db", NULL);
+  gchar *temporary_file = g_build_filename (ephy_dot_dir (), EPHY_HISTORY_FILE, NULL);
   /* Do nothing if the history file already exists. Safer than wiping
    * it out. */
   if (g_file_test (temporary_file, G_FILE_TEST_EXISTS)) {
-    g_warning ("Did not migrate Epiphany's history, the ephy-history.db file already exists");
+    g_warning ("Did not migrate Epiphany's history, the %s file already exists", EPHY_HISTORY_FILE);
     g_free (temporary_file);
     return;
   }
@@ -620,7 +619,7 @@ migrate_history ()
 }
 
 static void
-migrate_tabs_visibility ()
+migrate_tabs_visibility (void)
 {
   gboolean always_show_tabs;
 
@@ -670,7 +669,7 @@ migrate_profile (const char *old_dir,
 }
 
 static void
-migrate_profile_gnome2_to_xdg ()
+migrate_profile_gnome2_to_xdg (void)
 {
   char *old_dir;
   char *new_dir;
@@ -723,7 +722,7 @@ fix_desktop_file_and_return_new_location (const char *dir)
 }
 
 static void
-migrate_web_app_links ()
+migrate_web_app_links (void)
 {
   GList *apps, *p;
 
@@ -773,13 +772,13 @@ migrate_web_app_links ()
 }
 
 static void
-migrate_new_urls_table ()
+migrate_new_urls_table (void)
 {
   EphySQLiteConnection *history_database;
   char *filename;
   GError *error = NULL;
 
-  filename = g_build_filename (ephy_dot_dir (), "ephy-history.db", NULL);
+  filename = g_build_filename (ephy_dot_dir (), EPHY_HISTORY_FILE, NULL);
   history_database = ephy_sqlite_connection_new ();
   ephy_sqlite_connection_open (history_database, filename, &error);
 
@@ -831,7 +830,7 @@ const EphyProfileMigrator migrators[] = {
 };
 
 static gboolean
-ephy_migrator ()
+ephy_migrator (void)
 {
   int latest, i;
   EphyProfileMigrator m;
