@@ -24,14 +24,12 @@
 #include "ephy-session.h"
 
 #include "ephy-about-handler.h"
-#include "ephy-bookmarks-editor.h"
 #include "ephy-debug.h"
 #include "ephy-embed-container.h"
 #include "ephy-embed-utils.h"
 #include "ephy-embed.h"
 #include "ephy-file-helpers.h"
 #include "ephy-gui.h"
-#include "ephy-history-window.h"
 #include "ephy-prefs.h"
 #include "ephy-settings.h"
 #include "ephy-shell.h"
@@ -55,8 +53,6 @@ typedef struct
 
 struct _EphySessionPrivate
 {
-	GtkWidget *resume_window;
-
 	GQueue *queue;
 	guint queue_idle_id;
 
@@ -209,10 +205,9 @@ static void
 session_command_autoresume (EphySession *session,
 			    guint32 user_time)
 {
-	EphySessionPrivate *priv = session->priv;
 	GFile *saved_session_file;
 	char *saved_session_file_path;
-	gboolean crashed_session;
+	gboolean has_session_state;
 	EphyPrefsRestoreSessionPolicy policy;
 	EphyShell *shell;
 
@@ -221,7 +216,7 @@ session_command_autoresume (EphySession *session,
 	saved_session_file = get_session_file (SESSION_STATE);
 	saved_session_file_path = g_file_get_path (saved_session_file);
 	g_object_unref (saved_session_file);
-	crashed_session = g_file_test (saved_session_file_path, G_FILE_TEST_EXISTS);
+	has_session_state = g_file_test (saved_session_file_path, G_FILE_TEST_EXISTS);
 	
 	g_free (saved_session_file_path);
 
@@ -230,9 +225,8 @@ session_command_autoresume (EphySession *session,
 
 	shell = ephy_shell_get_default ();
 
-	if (crashed_session == FALSE ||
-	    policy == EPHY_PREFS_RESTORE_SESSION_POLICY_NEVER ||
-	    ephy_shell_get_n_windows (shell) > 0)
+	if (has_session_state == FALSE ||
+	    policy == EPHY_PREFS_RESTORE_SESSION_POLICY_NEVER)
 	{
 		/* If we are auto-resuming, and we never want to
 		 * restore the session, clobber the session state
@@ -243,32 +237,11 @@ session_command_autoresume (EphySession *session,
 		ephy_session_queue_command (session,
 					    EPHY_SESSION_CMD_MAYBE_OPEN_WINDOW,
 					    NULL, NULL, user_time, FALSE);
-
-		return;
 	}
-
-	if (priv->resume_window)
-	{
-		gtk_window_present_with_time (GTK_WINDOW (priv->resume_window),
-					      user_time);
-
-		return;
-	}
-
-	ephy_session_queue_command (session,
-				    EPHY_SESSION_CMD_LOAD_SESSION,
-				    SESSION_STATE, NULL, user_time, TRUE);
-}
-
-static void
-session_command_open_bookmarks_editor (EphySession *session,
-				       guint32 user_time)
-{
-	GtkWidget *editor;
-
-	editor = ephy_shell_get_bookmarks_editor (ephy_shell_get_default ());
-	
-	gtk_window_present_with_time (GTK_WINDOW (editor), user_time);
+	else if (ephy_shell_get_n_windows (shell) == 0)
+		ephy_session_queue_command (session,
+					    EPHY_SESSION_CMD_LOAD_SESSION,
+					    SESSION_STATE, NULL, user_time, TRUE);
 }
 
 static void
@@ -280,22 +253,22 @@ session_command_open_uris (EphySession *session,
 	EphyShell *shell;
 	EphyWindow *window;
 	EphyEmbed *embed;
-	EphySessionPrivate *priv;
 	EphyNewTabFlags flags = 0;
 	guint i;
 	gboolean new_windows_in_tabs;
-
-	priv = session->priv;
+	gboolean have_uris;
 
 	shell = ephy_shell_get_default ();
 
 	g_object_ref (shell);
 
-	window = ephy_shell_get_active_window (shell);
+	window = ephy_shell_get_main_window (shell);
 
 	new_windows_in_tabs = g_settings_get_boolean (EPHY_SETTINGS_MAIN,
 						      EPHY_PREFS_NEW_WINDOWS_IN_TABS);
 
+	have_uris = ! (g_strv_length (uris) == 1 && g_str_equal (uris[0], ""));
+
 	if (options != NULL && strstr (options, "external") != NULL)
 	{
 		flags |= EPHY_NEW_TAB_FROM_EXTERNAL;
@@ -306,12 +279,17 @@ session_command_open_uris (EphySession *session,
 		flags |= EPHY_NEW_TAB_IN_NEW_WINDOW;
 	}
 	else if ((options != NULL && strstr (options, "new-tab") != NULL) ||
-		 new_windows_in_tabs)
+		 (new_windows_in_tabs && have_uris))
 	{
 		flags |= EPHY_NEW_TAB_IN_EXISTING_WINDOW |
 			 EPHY_NEW_TAB_JUMP |
 			 EPHY_NEW_TAB_PRESENT_WINDOW;
 	}
+	else if (!have_uris)
+	{
+		window = NULL;
+		flags |= EPHY_NEW_TAB_IN_NEW_WINDOW;
+	}
 
 	for (i = 0; uris[i] != NULL; ++i)
 	{
@@ -337,29 +315,13 @@ session_command_open_uris (EphySession *session,
 #endif
 		}
 
-		/* For the first URI, if we have a valid recovery
-		 * window, reuse the already existing embed instead of
-		 * creating a new one, except if we still want to
-		 * present the option to resume a crashed session, in
-		 * that case use a new tab in the same window */
-		if (i == 0 && priv->resume_window != NULL)
-		{
-			EphyWebView *web_view;
-			
-			embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (priv->resume_window));
-			web_view = ephy_embed_get_web_view (embed);
-			ephy_web_view_load_url (web_view, url);
-		}
-		else
-		{
-			embed = ephy_shell_new_tab_full (shell, window,
-							 NULL /* parent tab */,
-							 request,
-							 flags | page_flags,
-							 EPHY_WEB_VIEW_CHROME_ALL,
-							 FALSE /* is popup? */,
-							 user_time);
-		}
+		embed = ephy_shell_new_tab_full (shell, window,
+						 NULL /* parent tab */,
+						 request,
+						 flags | page_flags,
+						 EPHY_WEB_VIEW_CHROME_ALL,
+						 FALSE /* is popup? */,
+						 user_time);
 
 		if (request)
 			g_object_unref (request);
@@ -391,9 +353,6 @@ session_command_dispatch (EphySession *session)
 		case EPHY_SESSION_CMD_LOAD_SESSION:
 			ephy_session_load (session, cmd->arg, cmd->user_time);
 			break;
-		case EPHY_SESSION_CMD_OPEN_BOOKMARKS_EDITOR:
-			session_command_open_bookmarks_editor (session, cmd->user_time);
-			break;
 		case EPHY_SESSION_CMD_OPEN_URIS:
 			session_command_open_uris (session, cmd->args, cmd->arg, cmd->user_time);
 			break;
@@ -580,7 +539,7 @@ ephy_session_close (EphySession *session)
 
 		session_command_queue_clear (session);
 
-		ephy_embed_shell_prepare_close (embed_shell);
+		ephy_embed_shell_prepare_close (ephy_embed_shell_get_default ());
 
 	}
 }
@@ -813,7 +772,8 @@ confirm_before_recover (EphyWindow* window, char* url, char* title)
 {
 	EphyEmbed *embed;
 
-	embed = ephy_shell_new_tab (ephy_shell, window, NULL, NULL,
+	embed = ephy_shell_new_tab (ephy_shell_get_default (),
+				    window, NULL, NULL,
 				    EPHY_NEW_TAB_IN_EXISTING_WINDOW |
 				    EPHY_NEW_TAB_APPEND_LAST);
 
@@ -826,11 +786,6 @@ parse_embed (xmlNodePtr child,
 	     EphyWindow *window,
 	     EphySession *session)
 {
-	EphySessionPrivate *priv = session->priv;
-	gboolean is_first_window;
-
-	is_first_window = window == EPHY_WINDOW (priv->resume_window);
-
 	while (child != NULL)
 	{
 		if (strcmp ((char *) child->name, "embed") == 0)
@@ -861,25 +816,11 @@ parse_embed (xmlNodePtr child,
 			{
 				recover_url = (char *) url;
 				
-				/* Reuse the window holding the recovery infobar instead of creating a new one. */
-				if (is_first_window == TRUE && window != NULL)
-				{
-					EphyWebView *web_view;
-					EphyEmbed *embed;
-
-					embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
-					web_view = ephy_embed_get_web_view (embed);
-					ephy_web_view_load_url (web_view, recover_url);
-
-					is_first_window = FALSE;
-				}
-				else
-				{
-					ephy_shell_new_tab (ephy_shell, window, NULL, recover_url,
-							    EPHY_NEW_TAB_IN_EXISTING_WINDOW |
-							    EPHY_NEW_TAB_OPEN_PAGE |
-							    EPHY_NEW_TAB_APPEND_LAST);
-				}
+				ephy_shell_new_tab (ephy_shell_get_default (),
+						    window, NULL, recover_url,
+						    EPHY_NEW_TAB_IN_EXISTING_WINDOW |
+						    EPHY_NEW_TAB_OPEN_PAGE |
+						    EPHY_NEW_TAB_APPEND_LAST);
 			}
 			else if (was_loading)
 			{
@@ -962,7 +903,6 @@ ephy_session_load_from_string (EphySession *session,
 	xmlNodePtr child;
 	EphyWindow *window;
 	GtkWidget *widget = NULL;
-	gboolean first_window_created = FALSE;
 	gboolean retval;
 
 	g_return_val_if_fail (EPHY_IS_SESSION (session), FALSE);
@@ -1004,13 +944,7 @@ ephy_session_load_from_string (EphySession *session,
 			xmlChar *tmp;
 			EphyEmbed *active_child;
 		    
-			if (first_window_created == FALSE && priv->resume_window != NULL)
-			{
-				window = EPHY_WINDOW (priv->resume_window);
-				first_window_created = TRUE;
-			}
-			else
-				window = ephy_window_new ();
+			window = ephy_window_new ();
 
 			widget = GTK_WIDGET (window);
 			restore_geometry (GTK_WINDOW (widget), child);
@@ -1037,7 +971,7 @@ ephy_session_load_from_string (EphySession *session,
 				}
 			}
 
-			if (ephy_embed_shell_get_mode (embed_shell) != EPHY_EMBED_SHELL_MODE_TEST)
+			if (ephy_embed_shell_get_mode (ephy_embed_shell_get_default ()) != EPHY_EMBED_SHELL_MODE_TEST)
 			{
 				active_child = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
 				gtk_widget_grab_focus (GTK_WIDGET (active_child));
@@ -1051,7 +985,6 @@ ephy_session_load_from_string (EphySession *session,
 	xmlFreeDoc (doc);
 
 	priv->dont_save = FALSE;
-	priv->resume_window = NULL;
 
 	ephy_session_save (session, SESSION_STATE);
 
@@ -1146,7 +1079,6 @@ ephy_session_queue_command (EphySession *session,
 
 			if ((command == EPHY_SESSION_CMD_LOAD_SESSION &&
 			     strcmp (cmd->arg, arg) == 0) ||
-			    command == EPHY_SESSION_CMD_OPEN_BOOKMARKS_EDITOR ||
 			    command == EPHY_SESSION_CMD_RESUME_SESSION)
 			{
 				cmd->user_time = user_time;
@@ -1178,10 +1110,4 @@ ephy_session_queue_command (EphySession *session,
 	session_command_queue_next (session);
 
 	g_application_hold (G_APPLICATION (ephy_shell_get_default ()));
-
-	if (priv->resume_window != NULL)
-	{
-		gtk_window_present_with_time (GTK_WINDOW (priv->resume_window),
-					      user_time);
-	}
 }
