@@ -47,7 +47,6 @@ typedef enum
 {
 	EPHY_SESSION_CMD_RESUME_SESSION,
 	EPHY_SESSION_CMD_LOAD_SESSION,
-	EPHY_SESSION_CMD_OPEN_BOOKMARKS_EDITOR,
 	EPHY_SESSION_CMD_OPEN_URIS,
 	EPHY_SESSION_CMD_MAYBE_OPEN_WINDOW,
 	EPHY_SESSION_CMD_MAYBE_OPEN_WINDOW_RESTORE,
@@ -70,8 +69,6 @@ struct _EphySessionClass
 
 GType		 ephy_session_get_type		(void);
 
-EphyWindow	*ephy_session_get_active_window	(EphySession *session);
-
 gboolean	 ephy_session_save		(EphySession *session,
 						 const char *filename);
 
