@@ -130,11 +130,7 @@ queue_commands (EphyShell *shell)
                                 EPHY_SESSION_CMD_RESUME_SESSION,
                                 NULL, NULL, ctx->user_time, TRUE);
 
-  if (ctx->startup_flags & EPHY_STARTUP_BOOKMARKS_EDITOR)
-    ephy_session_queue_command (session,
-                                EPHY_SESSION_CMD_OPEN_BOOKMARKS_EDITOR,
-                                NULL, NULL, ctx->user_time, FALSE);
-  else if (ctx->session_filename != NULL)
+  if (ctx->session_filename != NULL)
     ephy_session_queue_command (session,
                                 EPHY_SESSION_CMD_LOAD_SESSION,
                                 (const char *)ctx->session_filename, NULL,
@@ -168,6 +164,14 @@ new_window (GSimpleAction *action,
   window_cmd_file_new_window (NULL, NULL);
 }
 
+static void
+new_incognito_window (GSimpleAction *action,
+                      GVariant *parameter,
+                      gpointer user_data)
+{
+  window_cmd_file_new_incognito_window (NULL, NULL);
+}
+
 static void
 show_bookmarks (GSimpleAction *action,
                 GVariant *parameter,
@@ -205,9 +209,9 @@ show_about (GSimpleAction *action,
             GVariant *parameter,
             gpointer user_data)
 {
-  EphyWindow *window;
+  GtkWindow *window;
 
-  window = ephy_shell_get_active_window (ephy_shell);
+  window = gtk_application_get_active_window (GTK_APPLICATION (ephy_shell));
 
   window_cmd_help_about (NULL, GTK_WIDGET (window));
 }
@@ -222,6 +226,7 @@ quit_application (GSimpleAction *action,
 
 static GActionEntry app_entries[] = {
   { "new", new_window, NULL, NULL, NULL },
+  { "new-incognito", new_incognito_window, NULL, NULL, NULL },
   { "bookmarks", show_bookmarks, NULL, NULL, NULL },
   { "history", show_history, NULL, NULL, NULL },
   { "preferences", show_preferences, NULL, NULL, NULL },
@@ -600,7 +605,7 @@ download_started_cb (WebKitWebContext *web_context,
     return;
   }
 
-  window = ephy_shell_get_active_window (shell);
+  window = gtk_application_get_active_window (GTK_APPLICATION (shell));
 
   ed = ephy_download_new_for_download (download);
   ephy_download_set_window (ed, GTK_WIDGET (window));
@@ -612,6 +617,12 @@ ephy_shell_init (EphyShell *shell)
 {
   EphyShell **ptr = &ephy_shell;
 
+#ifdef HAVE_WEBKIT2
+  WebKitWebContext *web_context;
+  EphyEmbedShellMode mode;
+  char *favicon_db_path;
+#endif
+
   shell->priv = EPHY_SHELL_GET_PRIVATE (shell);
 
   /* globally accessible singleton */
@@ -621,9 +632,20 @@ ephy_shell_init (EphyShell *shell)
                              (gpointer *)ptr);
 
 #ifdef HAVE_WEBKIT2
-  g_signal_connect (webkit_web_context_get_default (), "download-started",
+  web_context = webkit_web_context_get_default ();
+  g_signal_connect (web_context, "download-started",
                     G_CALLBACK (download_started_cb),
                     shell);
+
+  /* Initialize the favicon cache as early as possible, or further
+     calls to webkit_web_context_get_favicon_database will fail. */
+  mode = ephy_embed_shell_get_mode (ephy_embed_shell_get_default ());
+  favicon_db_path = g_build_filename (mode == EPHY_EMBED_SHELL_MODE_PRIVATE ?
+                                      ephy_dot_dir () : g_get_user_cache_dir (),
+                                      g_get_prgname (), "icondatabase", NULL);
+
+  webkit_web_context_set_favicon_database_directory (web_context, favicon_db_path);
+  g_free (favicon_db_path);
 #endif
 }
 
@@ -707,6 +729,7 @@ ephy_shell_new_tab_full (EphyShell *shell,
                          gboolean is_popup,
                          guint32 user_time)
 {
+  EphyEmbedShell *embed_shell;
   EphyWindow *window;
   EphyEmbed *embed = NULL;
   gboolean fullscreen_lockdown = FALSE;
@@ -718,6 +741,8 @@ ephy_shell_new_tab_full (EphyShell *shell,
   gboolean is_empty = FALSE;
   int position = -1;
 
+  embed_shell = ephy_embed_shell_get_default ();
+
   if (flags & EPHY_NEW_TAB_OPEN_PAGE) open_page = TRUE;
   if (flags & EPHY_NEW_TAB_IN_NEW_WINDOW) in_new_window = TRUE;
   if (flags & EPHY_NEW_TAB_IN_EXISTING_WINDOW) in_new_window = FALSE;
@@ -818,7 +843,8 @@ ephy_shell_new_tab_full (EphyShell *shell,
     }
   }
 
-  if (flags & EPHY_NEW_TAB_PRESENT_WINDOW)
+  if (flags & EPHY_NEW_TAB_PRESENT_WINDOW &&
+      ephy_embed_shell_get_mode (EPHY_EMBED_SHELL (embed_shell)) != EPHY_EMBED_SHELL_MODE_TEST)
     gtk_window_present_with_time (GTK_WINDOW (window), user_time);
 
   return embed;
@@ -937,8 +963,11 @@ ephy_shell_get_bookmarks_editor (EphyShell *shell)
 GtkWidget *
 ephy_shell_get_history_window (EphyShell *shell)
 {
+  EphyEmbedShell *embed_shell;
   EphyHistoryService *service;
 
+  embed_shell = ephy_embed_shell_get_default ();
+
   if (shell->priv->history_window == NULL) {
     service = EPHY_HISTORY_SERVICE
       (ephy_embed_shell_get_global_history_service (embed_shell));
@@ -1041,21 +1070,42 @@ ephy_shell_get_n_windows (EphyShell *shell)
   return g_list_length (shell->priv->windows);
 }
 
-EphyWindow *
-ephy_shell_get_active_window (EphyShell *shell)
+EphyWindow*
+ephy_shell_get_main_window (EphyShell *shell)
 {
-  GList *l;
+  EphyWindow *window = NULL;
+  GList *windows;
+  GList *iter;
 
   g_return_val_if_fail (EPHY_IS_SHELL (shell), NULL);
 
-  for (l = shell->priv->windows; l != NULL; l = l->next) {
-    EphyEmbedContainer *window = EPHY_EMBED_CONTAINER (l->data);
+  /* Select the window with most tabs in the current workspace as the window to
+   * use for opening a new tab on, if that turns out to be the case.
+   */
+  windows = ephy_shell_get_windows (shell);
 
-    if (!ephy_embed_container_get_is_popup (window))
-      return EPHY_WINDOW (window);
+  for (iter = windows; iter != NULL; iter = iter->next) {
+    EphyWindow *candidate = EPHY_WINDOW (iter->data);
+    GtkNotebook *cur_notebook;
+    GtkNotebook *cand_notebook;
+
+    if (!ephy_window_is_on_current_workspace (candidate))
+      continue;
+
+    if (!window) {
+      window = candidate;
+      continue;
+    }
+
+    cur_notebook = GTK_NOTEBOOK (ephy_window_get_notebook (window));
+    cand_notebook =  GTK_NOTEBOOK (ephy_window_get_notebook (candidate));
+    if (gtk_notebook_get_n_pages (cand_notebook) > gtk_notebook_get_n_pages (cur_notebook))
+      window = candidate;
   }
 
-  return NULL;
+  g_list_free (windows);
+
+  return window;
 }
 
 gboolean
