@@ -21,7 +21,9 @@
 #include "config.h"
 #include "ephy-toolbar.h"
 
+#include "ephy-location-entry.h"
 #include "ephy-middle-clickable-button.h"
+#include "ephy-private.h"
 
 G_DEFINE_TYPE (EphyToolbar, ephy_toolbar, GTK_TYPE_TOOLBAR)
 
@@ -36,7 +38,8 @@ enum {
 static GParamSpec *object_properties[N_PROPERTIES] = { NULL, };
 
 struct _EphyToolbarPrivate {
-    EphyWindow *window;
+  EphyWindow *window;
+  GtkWidget *entry;
 };
 
 static void
@@ -82,11 +85,16 @@ ephy_toolbar_constructed (GObject *object)
   GtkAction *action;
   GtkToolItem *back_forward, *location_stop_reload, *tool_item;
   GtkWidget *tool_button, *box, *location, *toolbar;
+  GtkSizeGroup *size;
 
   G_OBJECT_CLASS (ephy_toolbar_parent_class)->constructed (object);
 
   toolbar = GTK_WIDGET (object);
 
+  /* Create a GtkSizeGroup to sync the height of the location entry, and
+   * the stop/reload button. */
+  size = gtk_size_group_new (GTK_SIZE_GROUP_VERTICAL);
+
   /* Set the MENUBAR style class so it's possible to drag the app
    * using the toolbar. */
   gtk_style_context_add_class (gtk_widget_get_style_context (toolbar),
@@ -127,15 +135,17 @@ ephy_toolbar_constructed (GObject *object)
   gtk_container_add (GTK_CONTAINER (back_forward), box);
   gtk_container_add (GTK_CONTAINER (toolbar), GTK_WIDGET (back_forward));
   gtk_widget_show_all (GTK_WIDGET (back_forward));
-  gtk_widget_set_margin_right (GTK_WIDGET (back_forward), 12);
+  if (gtk_widget_get_direction (GTK_WIDGET (back_forward)) == GTK_TEXT_DIR_RTL)
+    gtk_widget_set_margin_left (GTK_WIDGET (back_forward), 12);
+  else
+    gtk_widget_set_margin_right (GTK_WIDGET (back_forward), 12);
 
   /* Location and Reload/Stop */
   location_stop_reload = gtk_tool_item_new ();
   box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
 
   /* Location */
-  action = gtk_action_group_get_action (action_group, "Location");
-  location = gtk_action_create_tool_item (action);
+  priv->entry = location = ephy_location_entry_new ();
   gtk_box_pack_start (GTK_BOX (box), location,
                       TRUE, TRUE, 0);
   gtk_style_context_add_class (gtk_widget_get_style_context (box),
@@ -156,9 +166,18 @@ ephy_toolbar_constructed (GObject *object)
                            GTK_WIDGET (location_stop_reload),
                            "expand", TRUE,
                            NULL);
-                           
+  
   gtk_container_add (GTK_CONTAINER (toolbar), GTK_WIDGET (location_stop_reload));
-  gtk_widget_set_margin_right (GTK_WIDGET (location_stop_reload), 12);
+
+  gtk_size_group_add_widget (size, tool_button);
+  gtk_size_group_add_widget (size, location);
+  g_object_unref (size);
+
+  if (gtk_widget_get_direction (GTK_WIDGET (location_stop_reload)) == GTK_TEXT_DIR_RTL)
+    gtk_widget_set_margin_left (GTK_WIDGET (location_stop_reload), 12);
+  else
+    gtk_widget_set_margin_right (GTK_WIDGET (location_stop_reload), 12);
+
   gtk_widget_show_all (GTK_WIDGET (location_stop_reload));
 
   /* Page Menu */
@@ -214,3 +233,9 @@ ephy_toolbar_new (EphyWindow *window)
                                      "window", window,
                                      NULL));
 }
+
+GtkWidget *
+ephy_toolbar_get_location_entry (EphyToolbar *toolbar)
+{
+  return toolbar->priv->entry;
+}
