@@ -28,7 +28,6 @@
 #include "ephy-shell.h"
 #include "ephy-file-helpers.h"
 #include "ephy-gui.h"
-#include "ephy-state.h"
 #include "ephy-string.h"
 #include "ephy-debug.h"
 #include "ephy-time-helpers.h"
@@ -171,7 +170,7 @@ typedef struct
 
 #ifdef HAVE_WEBKIT2
 static WebKitCookieManager *
-get_cookie_manager ()
+get_cookie_manager (void)
 {
 	WebKitWebContext *web_context;
 
@@ -180,7 +179,7 @@ get_cookie_manager ()
 }
 #else
 static SoupCookieJar*
-get_cookie_jar ()
+get_cookie_jar (void)
 {
 	SoupSession* session;
 
@@ -243,7 +242,7 @@ got_network_passwords_list_cb (GnomeKeyringResult result,
 }
 
 static void
-_ephy_pdm_delete_all_passwords ()
+_ephy_pdm_delete_all_passwords (void)
 {
 	gnome_keyring_list_item_ids (GNOME_KEYRING_DEFAULT,
 				     got_network_passwords_list_cb,
@@ -307,6 +306,7 @@ clear_all_dialog_response_cb (GtkDialog *dialog,
 		{
 			EphyEmbedShell *shell;
 			EphyEmbedSingle *single;
+			WebKitFaviconDatabase *database;
 
 			shell = ephy_embed_shell_get_default ();
 
@@ -315,10 +315,11 @@ clear_all_dialog_response_cb (GtkDialog *dialog,
 			ephy_embed_single_clear_cache (single);
 
 #ifdef HAVE_WEBKIT2
-			/* TODO: Favicons */
+			database = webkit_web_context_get_favicon_database (webkit_web_context_get_default ());
 #else
-			webkit_favicon_database_clear (webkit_get_favicon_database ());
+			database = webkit_get_favicon_database ();
 #endif
+			webkit_favicon_database_clear (database);
 		}
 	}
 	gtk_widget_destroy (GTK_WIDGET (dialog));
@@ -1271,6 +1272,33 @@ passwords_show_toggled_cb (GtkWidget *button,
 	gtk_tree_view_column_set_visible (column, active);
 }
 
+static gboolean
+password_search_equal (GtkTreeModel *model,
+		       int column,
+		       const gchar *key,
+		       GtkTreeIter *iter,
+		       gpointer search_data)
+{
+	char *host, *user;
+	gboolean retval;
+
+	/* Note that this is function has to return FALSE for a *match* ! */
+
+	gtk_tree_model_get (model, iter, COL_PASSWORDS_HOST, &host, -1);
+	retval = strstr (host, key) == NULL;
+	g_free (host);
+	if (retval == FALSE)
+		return retval;
+
+	gtk_tree_model_get (model, iter, COL_PASSWORDS_USER, &user, -1);
+	retval = strstr (user, key) == NULL;
+	g_free (user);
+
+	return retval;
+}
+
+
+
 static void
 pdm_dialog_passwords_construct (PdmActionInfo *info)
 {
@@ -1353,6 +1381,11 @@ pdm_dialog_passwords_construct (PdmActionInfo *info)
 	gtk_tree_view_column_set_reorderable (column, TRUE);
 	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
 
+	/* Setup basic type ahead filtering */
+	gtk_tree_view_set_search_equal_func (treeview,
+					     (GtkTreeViewSearchEqualFunc) password_search_equal,
+					     dialog, NULL);
+
 	info->treeview = treeview;
 	setup_action (info);
 }
@@ -1577,7 +1610,7 @@ pdm_dialog_init (PdmDialog *dialog)
 	PdmActionInfo *cookies, *passwords;
 	GtkWidget *window;
 	gboolean has_private_profile = 
-		ephy_embed_shell_get_mode (embed_shell) == EPHY_EMBED_SHELL_MODE_PRIVATE;
+		ephy_embed_shell_get_mode (ephy_embed_shell_get_default ()) == EPHY_EMBED_SHELL_MODE_PRIVATE;
 
 
 	priv = dialog->priv = EPHY_PDM_DIALOG_GET_PRIVATE (dialog);
@@ -1656,7 +1689,7 @@ pdm_dialog_finalize (GObject *object)
 	PdmDialog *dialog = EPHY_PDM_DIALOG (object);
 	GObject *single;
 
-	single = ephy_embed_shell_get_embed_single (embed_shell);
+	single = ephy_embed_shell_get_embed_single (ephy_embed_shell_get_default ());
 
 	g_signal_handlers_disconnect_matched
 		(single, G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, object);
