@@ -46,7 +46,6 @@
 #include "ephy-session.h"
 #include "ephy-settings.h"
 #include "ephy-shell.h"
-#include "ephy-state.h"
 #include "ephy-string.h"
 #include "ephy-web-app-utils.h"
 #include "ephy-zoom.h"
@@ -65,6 +64,8 @@
 #include <webkit/webkit.h>
 #endif
 
+#define DEFAULT_ICON_SIZE 144
+
 void
 window_cmd_file_print (GtkAction *action,
 		       EphyWindow *window)
@@ -368,8 +369,10 @@ typedef struct {
 	GtkWidget *image;
 	GtkWidget *entry;
 	GtkWidget *spinner;
+	GtkWidget *spinner_box;
 	GtkWidget *box;
 	char *icon_href;
+	GdkRGBA icon_rgba;
 } EphyApplicationDialogData;
 
 static void
@@ -379,19 +382,155 @@ ephy_application_dialog_data_free (EphyApplicationDialogData *data)
 	g_slice_free (EphyApplicationDialogData, data);
 }
 
+static void
+rounded_rectangle (cairo_t *cr,
+                   gdouble  aspect,
+                   gdouble  x,
+                   gdouble  y,
+                   gdouble  corner_radius,
+                   gdouble  width,
+                   gdouble  height)
+{
+        gdouble radius;
+        gdouble degrees;
+
+        radius = corner_radius / aspect;
+        degrees = G_PI / 180.0;
+
+        cairo_new_sub_path (cr);
+        cairo_arc (cr,
+                   x + width - radius,
+                   y + radius,
+                   radius,
+                   -90 * degrees,
+                   0 * degrees);
+        cairo_arc (cr,
+                   x + width - radius,
+                   y + height - radius,
+                   radius,
+                   0 * degrees,
+                   90 * degrees);
+        cairo_arc (cr,
+                   x + radius,
+                   y + height - radius,
+                   radius,
+                   90 * degrees,
+                   180 * degrees);
+        cairo_arc (cr,
+                   x + radius,
+                   y + radius,
+                   radius,
+                   180 * degrees,
+                   270 * degrees);
+        cairo_close_path (cr);
+}
+
+static GdkPixbuf *
+frame_pixbuf (GdkPixbuf  *pixbuf,
+	      GdkRGBA    *rgba,
+	      int         width,
+	      int         height)
+{
+	GdkPixbuf *framed;
+	cairo_surface_t *surface;
+	cairo_t *cr;
+	int frame_width;
+	int radius;
+
+	surface = cairo_image_surface_create (CAIRO_FORMAT_ARGB32,
+					      width, height);
+	cr = cairo_create (surface);
+
+	frame_width = 0;
+	radius = 20;
+
+	rounded_rectangle (cr,
+			   1.0,
+			   frame_width + 0.5,
+			   frame_width + 0.5,
+			   radius,
+			   width - frame_width * 2 - 1,
+			   height - frame_width * 2 - 1);
+	if (rgba != NULL)
+			cairo_set_source_rgba (cr,
+					       rgba->red,
+					       rgba->green,
+					       rgba->blue,
+					       rgba->alpha);
+	else
+		cairo_set_source_rgba (cr, 0.5, 0.5, 0.5, 0.3);
+	cairo_fill_preserve (cr);
+
+	if (pixbuf != NULL) {
+		GdkPixbuf *scaled;
+		int w;
+		int h;
+
+		w = gdk_pixbuf_get_width (pixbuf);
+		h = gdk_pixbuf_get_height (pixbuf);
+
+		if (w < 48 || h < 48) {
+			scaled = gdk_pixbuf_scale_simple (pixbuf, w * 3, h * 3, GDK_INTERP_NEAREST);
+		} else if (w > width || h > height) {
+			double ws, hs, s;
+
+			ws = (double) width / w;
+			hs = (double) height / h;
+			s = MIN (ws, hs);
+			scaled = gdk_pixbuf_scale_simple (pixbuf, w * s, h * s, GDK_INTERP_BILINEAR);
+		} else {
+			scaled = g_object_ref (pixbuf);
+		}
+
+		w = gdk_pixbuf_get_width (scaled);
+		h = gdk_pixbuf_get_height (scaled);
+
+		gdk_cairo_set_source_pixbuf (cr, scaled,
+					     (width - w) / 2,
+					     (height - h) / 2);
+		g_object_unref (scaled);
+		cairo_fill (cr);
+	}
+
+	framed = gdk_pixbuf_get_from_surface (surface, 0, 0, width, height);
+	cairo_destroy (cr);
+	cairo_surface_destroy (surface);
+
+	return framed;
+}
+
 static void
 take_page_snapshot_and_set_image (EphyApplicationDialogData *data)
 {
 	GdkPixbuf *snapshot;
+	GdkPixbuf *framed;
 	int x, y, w, h;
 
 	x = y = 0;
-	w = h = 128; /* GNOME hi-res icon size. */
+	w = h = DEFAULT_ICON_SIZE;
 
 	snapshot = ephy_web_view_get_snapshot (data->view, x, y, w, h);
-
-	gtk_image_set_from_pixbuf (GTK_IMAGE (data->image), snapshot);
+	framed = frame_pixbuf (snapshot, NULL, w, h);
 	g_object_unref (snapshot);
+	gtk_image_set_from_pixbuf (GTK_IMAGE (data->image), framed);
+	g_object_unref (framed);
+}
+
+static void
+set_app_icon_from_filename (EphyApplicationDialogData *data,
+			    const char *filename)
+{
+	GdkPixbuf *pixbuf;
+	GdkPixbuf *framed;
+
+	pixbuf = gdk_pixbuf_new_from_file (filename, NULL);
+	if (pixbuf == NULL)
+		return;
+
+	framed = frame_pixbuf (pixbuf, &data->icon_rgba, DEFAULT_ICON_SIZE, DEFAULT_ICON_SIZE);
+	g_object_unref (pixbuf);
+	gtk_image_set_from_pixbuf (GTK_IMAGE (data->image), framed);
+	g_object_unref (framed);
 }
 
 #ifdef HAVE_WEBKIT2
@@ -401,8 +540,10 @@ download_finished_cb (WebKitDownload *download,
 {
 	char *filename;
 
+	gtk_widget_show (data->image);
+
 	filename = g_filename_from_uri (webkit_download_get_destination (download), NULL, NULL);
-	gtk_image_set_from_file (GTK_IMAGE (data->image), filename);
+	set_app_icon_from_filename (data, filename);
 	g_free (filename);
 }
 
@@ -411,6 +552,8 @@ download_failed_cb (WebKitDownload *download,
 		    GError *error,
 		    EphyApplicationDialogData *data)
 {
+	gtk_widget_show (data->image);
+
 	g_signal_handlers_disconnect_by_func (download, download_finished_cb, data);
 	/* Something happened, default to a page snapshot. */
 	take_page_snapshot_and_set_image (data);
@@ -424,12 +567,14 @@ download_status_changed_cb (WebKitDownload *download,
 	WebKitDownloadStatus status = webkit_download_get_status (download);
 	char *filename;
 
+	gtk_widget_show (data->image);
+
 	switch (status)
 	{
 	case WEBKIT_DOWNLOAD_STATUS_FINISHED:
 		filename = g_filename_from_uri (webkit_download_get_destination_uri (download),
 						   NULL, NULL);
-		gtk_image_set_from_file (GTK_IMAGE (data->image), filename);
+		set_app_icon_from_filename (data, filename);
 		g_free (filename);
 		break;
 	case WEBKIT_DOWNLOAD_STATUS_ERROR:
@@ -486,46 +631,124 @@ download_icon_and_set_image (EphyApplicationDialogData *data)
 #endif
 }
 
+
 static void
 fill_default_application_image (EphyApplicationDialogData *data)
 {
-#ifdef HAVE_WEBKIT2
-	/* TODO: DOM Bindindgs */
-#else
-	WebKitDOMDocument *document;
-	WebKitDOMNodeList *links;
-	gulong length, i;
+	gboolean res;
 
-	document = webkit_web_view_get_dom_document (WEBKIT_WEB_VIEW (data->view));
-	links = webkit_dom_document_get_elements_by_tag_name (document, "link");
-	length = webkit_dom_node_list_get_length (links);
+	data->icon_rgba.red = 0.5;
+	data->icon_rgba.green = 0.5;
+	data->icon_rgba.blue = 0.5;
+	data->icon_rgba.alpha = 0.3;
 
-	for (i = 0; i < length; i++)
+	res = ephy_web_view_get_best_icon (WEBKIT_WEB_VIEW (data->view),
+					   &data->icon_href,
+					   &data->icon_rgba);
+	if (res)
 	{
-		char *rel;
-		WebKitDOMNode *node = webkit_dom_node_list_item (links, i);
-		rel = webkit_dom_html_link_element_get_rel (WEBKIT_DOM_HTML_LINK_ELEMENT (node));
-		/* TODO: support more than one possible icon. */
-		if (g_strcmp0 (rel, "apple-touch-icon") == 0 ||
-		    g_strcmp0 (rel, "apple-touch-icon-precomposed") == 0)
+		download_icon_and_set_image (data);
+	}
+	else
+	{
+		gtk_widget_show (data->image);
+		take_page_snapshot_and_set_image (data);
+	}
+}
+
+typedef struct {
+	const char *host;
+	const char *name;
+} SiteInfo;
+
+static SiteInfo sites[] = {
+	{ "www.facebook.com", "Facebook" },
+	{ "twitter.com", "Twitter" },
+	{ "gmail.com", "GMail" },
+	{ "plus.google.com", "Google+" },
+	{ "youtube.com", "YouTube" },
+};
+
+static char *
+get_special_case_application_title_for_host (const char *host)
+{
+	char *title = NULL;
+	int i;
+
+	for (i = 0; i < G_N_ELEMENTS (sites) && title == NULL; i++)
+	{
+		SiteInfo *info = &sites[i];
+		if (strcmp (host, info->host) == 0)
 		{
-			data->icon_href = webkit_dom_html_link_element_get_href (WEBKIT_DOM_HTML_LINK_ELEMENT (node));
-			download_icon_and_set_image (data);
-			g_free (rel);
-			return;
+			title = g_strdup (info->name);
 		}
 	}
-#endif
-	/* If we make it here, no "apple-touch-icon" link was
-	 * found. Take a snapshot of the page. */
-	take_page_snapshot_and_set_image (data);
+
+	return title;
 }
 
 static void
 fill_default_application_title (EphyApplicationDialogData *data)
 {
-	const char *title = ephy_web_view_get_title (data->view);
+	char *title = NULL;
+#ifdef HAVE_WEBKIT2
+	/* TODO: DOM Bindindgs */
+#else
+	WebKitDOMDocument *document;
+	WebKitDOMNodeList *metas;
+	gulong length, i;
+
+	document = webkit_web_view_get_dom_document (WEBKIT_WEB_VIEW (data->view));
+	metas = webkit_dom_document_get_elements_by_tag_name (document, "meta");
+	length = webkit_dom_node_list_get_length (metas);
+
+	for (i = 0; i < length && title == NULL; i++)
+	{
+		char *name;
+		char *property;
+		WebKitDOMNode *node = webkit_dom_node_list_item (metas, i);
+
+		name = webkit_dom_html_meta_element_get_name (WEBKIT_DOM_HTML_META_ELEMENT (node));
+		property = webkit_dom_element_get_attribute (WEBKIT_DOM_ELEMENT (node), "property");
+		if (g_strcmp0 (name, "application-name") == 0
+		    || g_strcmp0 (property, "og:site_name") == 0)
+		{
+			title = webkit_dom_html_meta_element_get_content (WEBKIT_DOM_HTML_META_ELEMENT (node));
+		}
+		g_free (property);
+		g_free (name);
+	}
+#endif
+
+	if (title == NULL)
+	{
+		SoupURI *uri;
+		const char *host;
+
+		uri = soup_uri_new (webkit_web_view_get_uri (WEBKIT_WEB_VIEW (data->view)));
+		host = soup_uri_get_host (uri);
+
+		if (host != NULL && host[0] != '\0')
+			title = get_special_case_application_title_for_host (host);
+
+		if (title == NULL)
+		{
+			if (g_str_has_prefix (host, "www."))
+				title = g_strdup (host + strlen ("www."));
+			else
+				title = g_strdup (host);
+		}
+
+		soup_uri_free (uri);
+	}
+
+	if (title == NULL)
+	{
+		title = g_strdup (ephy_web_view_get_title (data->view));
+	}
+
 	gtk_entry_set_text (GTK_ENTRY (data->entry), title);
+	g_free (title);
 }
 
 static void
@@ -534,10 +757,7 @@ notify_launch_cb (NotifyNotification *notification,
 		  gpointer user_data)
 {
 	char * desktop_file = user_data;
-	/* A gross hack to be able to launch epiphany from within
-	 * Epiphany. Might be a good idea to figure out a better
-	 * solution... */
-	g_unsetenv (EPHY_UUID_ENVVAR);
+
 	ephy_file_launch_desktop_file (desktop_file, NULL, 0, NULL);
 	g_free (desktop_file);
 }
@@ -631,8 +851,14 @@ window_cmd_file_save_as_application (GtkAction *action,
 {
 	EphyEmbed *embed;
 	GtkWidget *dialog, *box, *image, *entry, *content_area;
+	GtkWidget *label;
+	GtkWidget *spinner;
+	GtkWidget *alignment;
 	EphyWebView *view;
 	EphyApplicationDialogData *data;
+	GdkPixbuf *pixbuf;
+	GtkStyleContext *context;
+	char *markup;
 
 	embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
 	g_return_if_fail (embed != NULL);
@@ -642,7 +868,7 @@ window_cmd_file_save_as_application (GtkAction *action,
 	/* Show dialog with icon, title. */
 	dialog = gtk_dialog_new_with_buttons (_("Create Web Application"),
 					      GTK_WINDOW (window),
-					      0,
+					      GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
 					      GTK_STOCK_CANCEL,
 					      GTK_RESPONSE_CANCEL,
 					      _("C_reate"),
@@ -650,24 +876,53 @@ window_cmd_file_save_as_application (GtkAction *action,
 					      NULL);
 
 	content_area = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
-	gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
-	gtk_box_set_spacing (GTK_BOX (content_area), 14); /* 14 + 2 * 5 = 24 */
+	gtk_container_set_border_width (GTK_CONTAINER (dialog), 10);
 
-	box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 5);
+	box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 5);
 	gtk_container_add (GTK_CONTAINER (content_area), box);
+	gtk_container_set_border_width (GTK_CONTAINER (box), 5);
 
 	image = gtk_image_new ();
-	gtk_widget_set_size_request (image, 128, 128);
+	gtk_widget_set_no_show_all (image, TRUE);
+	gtk_widget_set_size_request (image, DEFAULT_ICON_SIZE, DEFAULT_ICON_SIZE);
+	gtk_widget_set_margin_bottom (image, 10);
 	gtk_container_add (GTK_CONTAINER (box), image);
+	pixbuf = frame_pixbuf (NULL, NULL, DEFAULT_ICON_SIZE, DEFAULT_ICON_SIZE);
+	gtk_image_set_from_pixbuf (GTK_IMAGE (image), pixbuf);
+	g_object_unref (pixbuf);
+
+	alignment = gtk_alignment_new (0.5, 0.5, 0, 0);
+	gtk_widget_set_no_show_all (alignment, TRUE);
+	gtk_widget_set_size_request (alignment, DEFAULT_ICON_SIZE, DEFAULT_ICON_SIZE);
+	gtk_container_add (GTK_CONTAINER (box), alignment);
+	gtk_widget_show (alignment);
+
+	spinner = gtk_spinner_new ();
+	gtk_widget_set_size_request (spinner, 22, 22);
+	gtk_spinner_start (GTK_SPINNER (spinner));
+	gtk_container_add (GTK_CONTAINER (alignment), spinner);
+	gtk_widget_show (spinner);
 
 	entry = gtk_entry_new ();
 	gtk_entry_set_activates_default (GTK_ENTRY (entry), TRUE);
-	gtk_box_pack_end (GTK_BOX (box), entry, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (box), entry, FALSE, FALSE, 0);
+
+	markup = g_strdup_printf ("<small>%s</small>", webkit_web_view_get_uri (WEBKIT_WEB_VIEW (view)));
+	label = gtk_label_new (NULL);
+	gtk_label_set_markup (GTK_LABEL (label), markup);
+	g_free (markup);
+	gtk_box_pack_end (GTK_BOX (box), label, FALSE, FALSE, 0);
+	context = gtk_widget_get_style_context (label);
+	gtk_style_context_add_class (context, "dim-label");
 
 	data = g_slice_new0 (EphyApplicationDialogData);
 	data->view = view;
 	data->image = image;
 	data->entry = entry;
+	data->spinner = spinner;
+	data->spinner_box = alignment;
+
+	g_object_bind_property (image, "visible", data->spinner_box, "visible", G_BINDING_INVERT_BOOLEAN);
 
 	fill_default_application_image (data);
 	fill_default_application_title (data);
@@ -690,7 +945,7 @@ window_cmd_file_work_offline (GtkAction *action,
 	EphyEmbedSingle *single;
 	gboolean offline;
 
-	single = EPHY_EMBED_SINGLE (ephy_embed_shell_get_embed_single (embed_shell));
+	single = EPHY_EMBED_SINGLE (ephy_embed_shell_get_embed_single (ephy_embed_shell_get_default ()));
 	offline = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
 	ephy_embed_single_set_network_status (single, !offline);
 #endif
@@ -722,7 +977,7 @@ void
 window_cmd_file_quit (GtkAction *action,
 		      EphyWindow *window)
 {
-	if (ephy_shell_close_all_windows (ephy_shell))
+	if (ephy_shell_close_all_windows (ephy_shell_get_default ()))
 		g_application_quit (g_application_get_default ());
 }
 
@@ -730,8 +985,19 @@ void
 window_cmd_file_new_window (GtkAction *action,
 			    EphyWindow *window)
 {
-	ephy_shell_new_tab (ephy_shell, NULL, NULL, NULL,
-			    EPHY_NEW_TAB_IN_NEW_WINDOW | EPHY_NEW_TAB_HOME_PAGE);
+	ephy_shell_new_tab (ephy_shell_get_default (),
+			    NULL, NULL, NULL,
+			    EPHY_NEW_TAB_IN_NEW_WINDOW |
+			    EPHY_NEW_TAB_HOME_PAGE);
+}
+
+void
+window_cmd_file_new_incognito_window (GtkAction *action,
+				      EphyWindow *window)
+{
+	char *str = g_strdup_printf ("epiphany --incognito-mode --profile %s", ephy_dot_dir ());
+	g_spawn_command_line_async (str, NULL);
+	g_free (str);
 }
 
 void
@@ -952,7 +1218,7 @@ window_cmd_edit_bookmarks (GtkAction *action,
 {
 	GtkWidget *bwindow;
 	
-	bwindow = ephy_shell_get_bookmarks_editor (ephy_shell);
+	bwindow = ephy_shell_get_bookmarks_editor (ephy_shell_get_default ());
 	gtk_window_present (GTK_WINDOW (bwindow));
 }
 
@@ -962,7 +1228,7 @@ window_cmd_edit_history (GtkAction *action,
 {
 	GtkWidget *hwindow;
 	
-	hwindow = ephy_shell_get_history_window (ephy_shell);
+	hwindow = ephy_shell_get_history_window (ephy_shell_get_default ());
 	gtk_window_present (GTK_WINDOW (hwindow));
 }
 
@@ -972,7 +1238,7 @@ window_cmd_edit_preferences (GtkAction *action,
 {
 	EphyDialog *dialog;
 	
-	dialog = EPHY_DIALOG (ephy_shell_get_prefs_dialog (ephy_shell));
+	dialog = EPHY_DIALOG (ephy_shell_get_prefs_dialog (ephy_shell_get_default ()));
 	
 	ephy_dialog_show (dialog);
 }
@@ -983,7 +1249,7 @@ window_cmd_edit_personal_data (GtkAction *action,
 {
 	PdmDialog *dialog;
 	
-	dialog = EPHY_PDM_DIALOG (ephy_shell_get_pdm_dialog (ephy_shell));
+	dialog = EPHY_PDM_DIALOG (ephy_shell_get_pdm_dialog (ephy_shell_get_default ()));
 	/* FIXME?: pdm_dialog_open is supposed to scroll to the host passed
 	 * as second parameters in the cookies tab. Honestly I think this
 	 * has been broken for a while. In any case it's probably not
