@@ -30,6 +30,7 @@
 
 #include <glib.h>
 #include <glib/gstdio.h>
+#include <gdk/gdkx.h>
 #include <gtk/gtk.h>
 #include <string.h>
 
@@ -42,7 +43,7 @@ const char *session_data =
 "</session>";
 
 static void
-test_ephy_session_load ()
+test_ephy_session_load (void)
 {
     EphySession *session;
     gboolean ret;
@@ -50,13 +51,13 @@ test_ephy_session_load ()
     EphyEmbed *embed;
     EphyWebView *view;
 
-    session = EPHY_SESSION (ephy_shell_get_session (ephy_shell));
+    session = EPHY_SESSION (ephy_shell_get_session (ephy_shell_get_default ()));
     g_assert (session);
 
     ret = ephy_session_load_from_string (session, session_data, -1, 0);
     g_assert (ret);
 
-    l = ephy_shell_get_windows (ephy_shell);
+    l = ephy_shell_get_windows (ephy_shell_get_default ());
     g_assert (l);
     g_assert_cmpint (g_list_length (l), ==, 1);
 
@@ -77,7 +78,7 @@ const char *session_data_empty =
 "";
 
 static void
-test_ephy_session_load_empty_session ()
+test_ephy_session_load_empty_session (void)
 {
     EphySession *session;
     gboolean ret;
@@ -85,7 +86,7 @@ test_ephy_session_load_empty_session ()
     EphyEmbed *embed;
     EphyWebView *view;
 
-    session = EPHY_SESSION (ephy_shell_get_session (ephy_shell));
+    session = EPHY_SESSION (ephy_shell_get_session (ephy_shell_get_default ()));
     g_assert (session);
 
     ret = ephy_session_load_from_string (session, session_data_empty, -1, 0);
@@ -98,7 +99,7 @@ test_ephy_session_load_empty_session ()
     while (g_main_context_pending (NULL))
       g_main_context_iteration (NULL, FALSE);
 
-    l = ephy_shell_get_windows (ephy_shell);
+    l = ephy_shell_get_windows (ephy_shell_get_default ());
     g_assert (l);
     g_assert_cmpint (g_list_length (l), ==, 1);
 
@@ -127,7 +128,7 @@ const char *session_data_many_windows =
 "</session>";
 
 static void
-test_ephy_session_load_many_windows ()
+test_ephy_session_load_many_windows (void)
 {
     EphySession *session;
     gboolean ret;
@@ -135,13 +136,13 @@ test_ephy_session_load_many_windows ()
     EphyEmbed *embed;
     EphyWebView *view;
 
-    session = EPHY_SESSION (ephy_shell_get_session (ephy_shell));
+    session = EPHY_SESSION (ephy_shell_get_session (ephy_shell_get_default ()));
     g_assert (session);
 
     ret = ephy_session_load_from_string (session, session_data_many_windows, -1, 0);
     g_assert (ret);
 
-    l = ephy_shell_get_windows (ephy_shell);
+    l = ephy_shell_get_windows (ephy_shell_get_default ());
     g_assert (l);
     g_assert_cmpint (g_list_length (l), ==, 2);
 
@@ -152,6 +153,102 @@ test_ephy_session_load_many_windows ()
       g_assert (view);
       g_assert_cmpstr (ephy_web_view_get_address (view), ==, "ephy-about:epiphany");
     }
+
+    /* FIXME: See comments above. */
+    for (p = l; p; p = p->next)
+      gtk_widget_destroy (GTK_WIDGET (p->data));
+}
+
+static void
+open_uris_after_loading_session (const char** uris, int final_num_windows)
+{
+    EphySession *session;
+    gboolean ret;
+    GList *l, *p;
+    EphyEmbed *embed;
+    EphyWebView *view;
+    guint32 user_time;
+
+    session = EPHY_SESSION (ephy_shell_get_session (ephy_shell_get_default ()));
+    g_assert (session);
+
+    user_time = gdk_x11_display_get_user_time (gdk_display_get_default ());
+
+    ret = ephy_session_load_from_string (session, session_data_many_windows, -1, 0);
+    g_assert (ret);
+
+    l = ephy_shell_get_windows (ephy_shell_get_default ());
+    g_assert (l);
+    g_assert_cmpint (g_list_length (l), ==, 2);
+
+    for (p = l; p; p = p->next) {
+      embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (p->data));
+      g_assert (embed);
+      view = ephy_embed_get_web_view (embed);
+      g_assert (view);
+      g_assert_cmpstr (ephy_web_view_get_address (view), ==, "ephy-about:epiphany");
+    }
+
+    /* Causing a session load here should not create new windows, since we
+     * already have some.
+     */
+    ephy_session_save (session, "type:session_state");
+
+    ephy_session_queue_command (session,
+                                EPHY_SESSION_CMD_RESUME_SESSION,
+                                "type:session_state",
+                                NULL,
+                                user_time,
+                                FALSE);
+
+    /* Ensure the queue is processed. */
+    while (gtk_events_pending ())
+        gtk_main_iteration_do (FALSE);
+
+    l = ephy_shell_get_windows (ephy_shell_get_default ());
+    g_assert (l);
+    g_assert_cmpint (g_list_length (l), ==, 2);
+
+    /* We should still have only 2 windows after the session load
+     * command - it should bail after noticing there are windows
+     * already.
+     */
+    ephy_session_queue_command (session,
+                                EPHY_SESSION_CMD_OPEN_URIS,
+                                NULL,
+                                uris,
+                                user_time,
+                                FALSE);
+
+    while (gtk_events_pending ())
+        gtk_main_iteration_do (FALSE);
+
+    /* We should still have 2 windows here, since the new URI should be
+     * in a new tab of an existing window.
+     */
+    l = ephy_shell_get_windows (ephy_shell_get_default ());
+    g_assert (l);
+    g_assert_cmpint (g_list_length (l), ==, final_num_windows);
+
+    /* FIXME: See comments above. */
+    for (p = l; p; p = p->next)
+      gtk_widget_destroy (GTK_WIDGET (p->data));
+}
+
+static void
+test_ephy_session_open_uri_after_loading_session (void)
+{
+    const char* uris[] = { "ephy-about:epiphany", NULL };
+
+    open_uris_after_loading_session (uris, 2);
+}
+
+static void
+test_ephy_session_open_empty_uri_forces_new_window (void)
+{
+    const char* uris[] = { "", NULL };
+
+    open_uris_after_loading_session (uris, 3);
 }
 
 int
@@ -172,9 +269,9 @@ main (int argc, char *argv[])
   }
 
   _ephy_shell_create_instance (EPHY_EMBED_SHELL_MODE_TEST);
-  g_assert (ephy_shell);
+  g_assert (ephy_shell_get_default ());
 
-  g_application_register (G_APPLICATION (ephy_shell), NULL, NULL);
+  g_application_register (G_APPLICATION (ephy_shell_get_default ()), NULL, NULL);
 
   g_test_add_func ("/src/ephy-session/load",
                    test_ephy_session_load);
@@ -185,10 +282,16 @@ main (int argc, char *argv[])
   g_test_add_func ("/src/ephy-session/load-many-windows",
                    test_ephy_session_load_many_windows);
 
+  g_test_add_func ("/src/ephy-session/open-uri-after-loading_session",
+                   test_ephy_session_open_uri_after_loading_session);
+
+  g_test_add_func ("/src/ephy-session/open-empty-uri-forces-new-window",
+                   test_ephy_session_open_empty_uri_forces_new_window);
+
   ret = g_test_run ();
 
+  g_object_unref (ephy_shell_get_default ());
   ephy_file_helpers_shutdown ();
-  g_object_unref (ephy_shell);
 
   return ret;
 }
